@@ -122,7 +122,7 @@ def single_line(val):
     # quick and dirty validation for description pypa/setuptools#1390
     if '\n' in val:
         # TODO after 2021-07-31: Replace with `raise ValueError("newlines not allowed")`
-        warnings.UserWarning("newlines not allowed and will break in the future")
+        warnings.warn("newlines not allowed and will break in the future")
         val = val.replace('\n', ' ')
     return val
 
