@@ -1,20 +1,20 @@
 """develop tests
 """
 
-from __future__ import absolute_import, unicode_literals
-
 import os
 import site
 import sys
 import io
 import subprocess
+import platform
 
-from setuptools.extern import six
+from setuptools.command import test
 
 import pytest
 
 from setuptools.command.develop import develop
 from setuptools.dist import Distribution
+from setuptools.tests import ack_2to3
 from . import contexts
 from . import namespaces
 
@@ -31,7 +31,7 @@ INIT_PY = """print "foo"
 """
 
 
-@pytest.yield_fixture
+@pytest.fixture
 def temp_user(monkeypatch):
     with contexts.tempdir() as user_base:
         with contexts.tempdir() as user_site:
@@ -40,7 +40,7 @@ def temp_user(monkeypatch):
             yield
 
 
-@pytest.yield_fixture
+@pytest.fixture
 def test_env(tmpdir, temp_user):
     target = tmpdir
     foo = target.mkdir('foo')
@@ -60,8 +60,10 @@ class TestDevelop:
     in_virtualenv = hasattr(sys, 'real_prefix')
     in_venv = hasattr(sys, 'base_prefix') and sys.base_prefix != sys.prefix
 
-    @pytest.mark.skipif(in_virtualenv or in_venv,
+    @pytest.mark.skipif(
+        in_virtualenv or in_venv,
         reason="Cannot run when invoked in a virtualenv or venv")
+    @ack_2to3
     def test_2to3_user_mode(self, test_env):
         settings = dict(
             name='foo',
@@ -92,7 +94,7 @@ class TestDevelop:
         with io.open(fn) as init_file:
             init = init_file.read().strip()
 
-        expected = 'print("foo")' if six.PY3 else 'print "foo"'
+        expected = 'print("foo")'
         assert init == expected
 
     def test_console_scripts(self, tmpdir):
@@ -100,7 +102,8 @@ class TestDevelop:
         Test that console scripts are installed and that they reference
         only the project by name and not the current version.
         """
-        pytest.skip("TODO: needs a fixture to cause 'develop' "
+        pytest.skip(
+            "TODO: needs a fixture to cause 'develop' "
             "to be invoked without mutating environment.")
         settings = dict(
             name='foo',
@@ -121,6 +124,22 @@ class TestDevelop:
         # assert '0.0' not in foocmd_text
 
 
+class TestResolver:
+    """
+    TODO: These tests were written with a minimal understanding
+    of what _resolve_setup_path is intending to do. Come up with
+    more meaningful cases that look like real-world scenarios.
+    """
+    def test_resolve_setup_path_cwd(self):
+        assert develop._resolve_setup_path('.', '.', '.') == '.'
+
+    def test_resolve_setup_path_one_dir(self):
+        assert develop._resolve_setup_path('pkgs', '.', 'pkgs') == '../'
+
+    def test_resolve_setup_path_one_dir_trailing_slash(self):
+        assert develop._resolve_setup_path('pkgs/', '.', 'pkgs') == '../'
+
+
 class TestNamespaces:
 
     @staticmethod
@@ -132,12 +151,18 @@ class TestNamespaces:
             'develop',
             '--install-dir', str(target),
         ]
-        env = dict(PYTHONPATH=str(target))
         with src_dir.as_cwd():
-            subprocess.check_call(develop_cmd, env=env)
+            with test.test.paths_on_pythonpath([str(target)]):
+                subprocess.check_call(develop_cmd)
 
-    @pytest.mark.skipif(bool(os.environ.get("APPVEYOR")),
-        reason="https://github.com/pypa/setuptools/issues/851")
+    @pytest.mark.skipif(
+        bool(os.environ.get("APPVEYOR")),
+        reason="https://github.com/pypa/setuptools/issues/851",
+    )
+    @pytest.mark.skipif(
+        platform.python_implementation() == 'PyPy',
+        reason="https://github.com/pypa/setuptools/issues/1202",
+    )
     def test_namespace_package_importable(self, tmpdir):
         """
         Installing two packages sharing the same namespace, one installed
@@ -150,6 +175,8 @@ class TestNamespaces:
         target = tmpdir / 'packages'
         # use pip to install to the target directory
         install_cmd = [
+            sys.executable,
+            '-m',
             'pip',
             'install',
             str(pkg_A),
@@ -162,12 +189,13 @@ class TestNamespaces:
             sys.executable,
             '-c', 'import myns.pkgA; import myns.pkgB',
         ]
-        env = dict(PYTHONPATH=str(target))
-        subprocess.check_call(try_import, env=env)
+        with test.test.paths_on_pythonpath([str(target)]):
+            subprocess.check_call(try_import)
 
         # additionally ensure that pkg_resources import works
         pkg_resources_imp = [
             sys.executable,
             '-c', 'import pkg_resources',
         ]
-        subprocess.check_call(pkg_resources_imp, env=env)
+        with test.test.paths_on_pythonpath([str(target)]):
+            subprocess.check_call(pkg_resources_imp)
